@@ -1,25 +1,45 @@
-# tree.py: classes to handle and display the structure of a wxGlade app
-# $Id: tree.py,v 1.56 2007/08/07 12:13:44 agriggio Exp $
-# 
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""\
+Classes to handle and display the structure of a wxGlade app
 
-#from wxPython.wx import *
+@copyright: 2002-2007 Alberto Griggio
+@copyright_ 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
+
+import logging
+import os.path
+import sys
+import StringIO
+import time
+import types
 import wx
-from xml.sax.saxutils import quoteattr
-import misc, common, os.path
 
-try: set
-except NameError: from sets import Set as set
+import misc
+import common
+import compat
+import config
 
 
-class Tree:
+class SlotNode(object):
+    """\
+    To write a placeholder for an empty sizer slot
+    """
+
+    def write(self, outfile, tabs):
+        stmt = common.format_xml_tag(
+            u'object', '', tabs, **{'class': 'sizerslot'})
+        outfile.write(stmt)
+
+
+class Tree(object):
     """\
     A class to represent a hierarchy of widgets.
+
+    @ivar _logger: Class specific logging instance
     """
-    class Node:
+    class Node(object):
         __empty_win = None
+
         def __init__(self, widget=None, children=None):
             self.widget = widget
             self.children = children
@@ -33,19 +53,22 @@ class Tree:
                 if node.widget is not None:
                     # replace the just destroyed notebook with an empty window
                     pw = node.widget.property_window
-                    pw.SetTitle(_('Properties - <>'))
+                    pw.SetTitle(_('Properties - <%s>') % '')
                     if Tree.Node.__empty_win is None:
                         Tree.Node.__empty_win = wx.Window(pw, -1)
-                    pw.GetSizer().GetChildren()[0].SetWindow(
-                        Tree.Node.__empty_win)
-                    #wxNotebook(node.widget.property_window, -1))
+                    compat.SizerItem_SetWindow(
+                        pw.GetSizer().GetChildren()[0],
+                        Tree.Node.__empty_win,
+                        )
                     # call the widget's ``destructor''
                     node.widget.delete()
                 node.widget = None
             remove_rec(self)
-            try: self.parent.children.remove(self)
-            except: pass
-                    
+            try:
+                self.parent.children.remove(self)
+            except:
+                pass
+
         def __repr__(self):
             try: return self.widget.name
             except AttributeError: return repr(self.widget)
@@ -57,85 +80,129 @@ class Tree:
             import edit_sizers
             fwrite = outfile.write
             assert self.widget is not None
-            w = self.widget
-            classname = getattr(w, '_classname', w.__class__.__name__)
-            # ALB 2005-11-19: to disable custom class code generation
-            # (for panels...)
-            no_custom = ""
-            if getattr(w, 'no_custom_class', False):
-                no_custom = ' no_custom_class="1"'
-            fwrite('    ' * tabs + '<object class=%s name=%s base=%s%s>\n'
-                   % (quoteattr(w.klass), quoteattr(w.name),
-                      quoteattr(classname), no_custom))
-            for p in w.properties:
-                w.properties[p].write(outfile, tabs+1)
+            classname = getattr(self.widget, '_classname',
+                                self.widget.__class__.__name__)
+            # to disable custom class code generation (for panels...)
+            if getattr(self.widget, 'no_custom_class', False):
+                no_custom = u' no_custom_class="1"'
+            else:
+                no_custom = ""
+            outer_tabs = u'    ' * tabs
+            fwrite(u'%s<object %s %s %s%s>\n' % (
+                outer_tabs,
+                common.format_xml_attrs(**{'class': self.widget.klass}),
+                common.format_xml_attrs(name=self.widget.name),
+                common.format_xml_attrs(base=classname),
+                no_custom))
 
-            if class_names is not None:
-                class_names.add(w.klass)
+            for prop in self.widget.properties:
+                if not getattr(self.widget.properties[prop], '_omitter',
+                               None):
+                    self.write_by_omitter(self.widget, prop, outfile,
+                                          tabs + 1)
 
-            if isinstance(w, edit_sizers.SizerBase):
-                maxpos = len(w.children)
-                children = self.children is not None and self.children or []
-                tmp = {}
-                for c in children:
-                    tmp[c.widget.pos] = c
-                children = []
-                class SlotNode:
-                    def write(self, outfile, tabs):
-                        fwrite('    ' * tabs + '<object class="sizerslot" />\n')
-                for i in range(1, maxpos):
-                    children.append(tmp.get(i, SlotNode()))
-                #if self.children is not None:
-                #    for c in self.children:
-                for c in children:
-                    if hasattr(c, 'widget'):
-                        fwrite('    ' * (tabs+1) +
-                               '<object class="sizeritem">\n')
-                        sp = c.widget.sizer_properties
-                        for p in sp: sp[p].write(outfile, tabs+2)
-                        c.write(outfile, tabs+2, class_names)
-                        fwrite('    ' * (tabs+1) + '</object>\n')
+            if class_names is not None and \
+                            self.widget.__class__.__name__ != 'CustomWidget':
+                class_names.add(self.widget.klass)
+
+            # sort children by sizer position and add "sizerlots" for empty
+            # positions
+            if isinstance(self.widget, edit_sizers.SizerBase):
+                # number of sizer slots from parent object
+                sizer_slots = len(self.widget.children)
+
+                # children = self.children is not None and self.children or []
+                children = self.children or []
+
+                # create list of children / "sizerslots" based on sizer
+                # position
+                children_by_position = {}
+                for child in children:
+                    children_by_position[child.widget.pos] = child
+
+                # convert back to list
+                children = [children_by_position.get(pos, SlotNode())
+                            for pos in range(1, sizer_slots)]
+
+                for child in children:
+                    if hasattr(child, 'widget'):
+                        inner_xml = StringIO.StringIO()
+                        for prop in child.widget.sizer_properties.values():
+                            prop.write(inner_xml, tabs + 2)
+                        child.write(inner_xml, tabs + 2, class_names)
+                        stmt = common.format_xml_tag(
+                            u'object', inner_xml.getvalue(), tabs + 1,
+                            is_xml=True, **{'class': 'sizeritem'}
+                        )
+                        fwrite(stmt)
                     else:
-                        c.write(outfile, tabs+1)
+                        child.write(outfile, tabs + 1)
             elif self.children is not None:
-                for c in self.children: c.write(outfile, tabs+1, class_names)
-            fwrite('    ' * tabs + '</object>\n')
+                for child in self.children:
+                    child.write(outfile, tabs + 1, class_names)
+            fwrite(u'%s</object>\n' % outer_tabs)
+
+        def write_by_omitter(self, widget, name, file, tabs):
+            """\
+            Write to given output file:
+            - value of property
+            - values of properties that previous can block
+            - any omitter can be blocked as well
+            """
+            widget.properties[name].write(file, tabs)
+            if getattr(widget, 'get_property_blocking', None):
+                items = widget.get_property_blocking(name)
+                if items:
+                    for name in items:
+                        self.write_by_omitter(widget, name, file, tabs)
 
     # end of class Node
 
     def __init__(self, root=None, app=None):
+        # initialise instance logger
+        self._logger = logging.getLogger(self.__class__.__name__)
+
+        # initialise instance
         self.root = root
         if self.root is None: self.root = Tree.Node()
         self.current = self.root
-        self.app = app # reference to the app properties
-        self.names = {} # dictionary of names of the widgets: each entry is
+        self.app = app  # reference to the app properties
+        self.names = {}  # dictionary of names of the widgets: each entry is
                         # itself a dictionary, one for each toplevel widget...
 
     def _find_toplevel(self, node):
         assert node is not None, _("None node in _find_toplevel")
         if node.parent is self.root:
-            return node.parent
+            return node
         while node.parent is not self.root:
             node = node.parent
         return node
 
     def has_name(self, name, node=None):
         if node is None:
-            #print '\nname to check:', name
+            #self._logger.debug('name to check: %s', name)
             for n in self.names:
-                #print 'names of %s: %s' % (n.widget.name, self.names[n])
+                #self._logger.debug(
+                #    'names of %s: %s',
+                #    n.widget.name,
+                #    self.names[n]
+                #)
                 if name in self.names[n]:
                     return True
             return False
         else:
             node = self._find_toplevel(node)
-            #print '\nname to check:', name
-            #print 'names of node %s: %s' % (node.widget.name, self.names[node])
+            #self._logger.debug('name to check: %s', name)
+            #self._logger.debug(
+            #    'names of node %s: %s',
+            #    node.widget.name,
+            #    self.names[node]
+            #    )
             return name in self.names[node]
         #return self.names.has_key(name)
 
     def add(self, child, parent=None):
-        if parent is None: parent = self.root 
+        if parent is None: parent = self.root
         if parent.children is None: parent.children = []
         parent.children.append(child)
         child.parent = parent
@@ -184,46 +251,69 @@ class Tree:
 
     def write(self, outfile=None, tabs=0):
         """\
-        Writes the xml equivalent of this tree to the given output file
+        Writes the xml equivalent of this tree to the given output file.
+
+        This function writes unicode to the outfile.
         """
         if outfile is None:
-            import sys
             outfile = sys.stdout
-        from time import asctime
-        import common
-        outfile.write('<?xml version="1.0"?>\n<!-- generated by wxGlade %s ' \
-                      'on %s -->\n\n' % (common.version, asctime()))
-        outpath = os.path.expanduser(self.app.output_path.strip())
+        outfile.write(u'<?xml version="1.0"?>\n'
+                      u'<!-- generated by wxGlade %s on %s -->\n\n' %
+                      (config.version, time.asctime()))
+        outpath = os.path.normpath(
+            os.path.expanduser(self.app.output_path.strip()))
         name = self.app.get_name()
         klass = self.app.get_class()
-        option = str(self.app.codegen_opt)
+        option = self._to_digit_string(self.app.multiple_files)
         top_window = self.app.get_top_window()
         language = self.app.get_language()
         encoding = self.app.get_encoding()
-        use_gettext = str(int(self.app.use_gettext))
-        is_template = str(int(self.app.is_template))
-        overwrite = str(int(self.app.overwrite))
-        # ALB 2004-01-18
-        #use_new_namespace = str(int(self.app.get_use_new_namespace()))
-        use_new_namespace = str(int(not self.app.get_use_old_namespace()))
-        # ALB 2004-12-05
+        use_gettext = self._to_digit_string(self.app.use_gettext)
+        is_template = self._to_digit_string(self.app.is_template)
+        overwrite = self._to_digit_string(self.app.overwrite)
+        indent_amount = self._to_digit_string(self.app.indent_amount)
+        indent_symbol = self._to_digit_string(self.app.indent_mode)
+        if indent_symbol == '0':
+            indent_symbol = u"tab"
+        else:
+            indent_symbol = u"space"
+        source_ext = '.' + self.app.source_ext
+        header_ext = '.' + self.app.header_ext
         for_version = str(self.app.for_version)
-        outfile.write('<application path=%s name=%s class=%s option=%s ' \
-                      'language=%s top_window=%s encoding=%s ' \
-                      'use_gettext=%s overwrite=%s '
-                      'use_new_namespace=%s for_version=%s is_template=%s>\n' \
-                      % tuple([quoteattr(common._encode_to_xml(i)) for i in 
-                               (outpath, name, klass, option, language,
-                                top_window, encoding, use_gettext,
-                                overwrite, use_new_namespace, for_version,
-                                is_template)]))
+
+        attrs = {
+            'path': outpath,
+            'name': name,
+            'class': klass,
+            'option': option,
+            'language': language,
+            'top_window': top_window,
+            'encoding': encoding,
+            'use_gettext': use_gettext,
+            'overwrite': overwrite,
+            'use_new_namespace': 1,
+            'for_version': for_version,
+            'is_template': is_template,
+            'indent_amount': indent_amount,
+            'indent_symbol': indent_symbol,
+            'source_extension': source_ext,
+            'header_extension': header_ext,
+        }
+
+        inner_xml = StringIO.StringIO()
+
         if self.app.is_template and getattr(self.app, 'template_data', None):
-            self.app.template_data.write(outfile, tabs+1)
+            self.app.template_data.write(inner_xml, tabs + 1)
+
         class_names = set()
         if self.root.children is not None:
             for c in self.root.children:
-                c.write(outfile, tabs+1, class_names)
-        outfile.write('</application>\n')
+                c.write(inner_xml, tabs + 1, class_names)
+
+        stmt = common.format_xml_tag(
+            u'application', inner_xml.getvalue(), is_xml=True, **attrs)
+        outfile.write(stmt)
+
         return class_names
 
     def change_node(self, node, widget):
@@ -243,11 +333,38 @@ class Tree:
         if index is None: index = node.parent.children.index(node)
         if index >= new_pos:
             node.parent.children.insert(new_pos, node)
-            del node.parent.children[index+1]
+            del node.parent.children[index + 1]
         else:
             del node.parent.children[index]
             node.parent.children.insert(new_pos+1, node)
 
+    def _to_digit_string(self, value):
+        """\
+        Convert the given value to a digit string.
+
+        @param value: Value to convert
+        @type value: str|int|bool
+
+        @rtype: str
+        """
+        if isinstance(value, types.BooleanType):
+            if value:
+                return '1'
+            else:
+                return '0'
+        elif isinstance(value, types.IntType):
+            return '%s' % value
+        elif isinstance(value, types.StringTypes):
+            if value.isdigit():
+                return value
+
+        # log failures
+        logging.warning(
+            _('Failed to convert "%s" (type: %s) to a digit string'),
+            value, type(value))
+
+        return '%s' % value
+
 # end of class Tree
 
 
@@ -255,8 +372,19 @@ class WidgetTree(wx.TreeCtrl, Tree):
     """\
     Tree with the ability to display the hierarchy of widgets
     """
-    images = {} # dictionary of icons of the widgets displayed
+
+    images = {}
+    """\
+    Dictionary of icons of the widgets displayed
+    """
+
+    _logger = None
+    """\
+    Class specific logging instance
+    """
+
     def __init__(self, parent, application):
+        self._logger = logging.getLogger(self.__class__.__name__)
         id = wx.NewId()
         style = wx.TR_DEFAULT_STYLE|wx.TR_HAS_VARIABLE_ROW_HEIGHT
         if wx.Platform == '__WXGTK__':
@@ -265,11 +393,11 @@ class WidgetTree(wx.TreeCtrl, Tree):
             style &= ~wx.TR_ROW_LINES
         wx.TreeCtrl.__init__(self, parent, id, style=style)
         root_node = Tree.Node(application)
-        self.cur_widget = None # reference to the selected widget
+        self.cur_widget = None  # reference to the selected widget
         Tree.__init__(self, root_node, application)
         image_list = wx.ImageList(21, 21)
-        image_list.Add(wx.Bitmap(os.path.join(common.wxglade_path,
-                                             'icons/application.xpm'),
+        image_list.Add(wx.Bitmap(os.path.join(config.icons_path,
+                                             'application.xpm'),
                                 wx.BITMAP_TYPE_XPM))
         for w in WidgetTree.images:
 ##             WidgetTree.images[w] = image_list.Add(wx.Bitmap(
@@ -279,46 +407,47 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.AssignImageList(image_list)
         root_node.item = self.AddRoot(_('Application'), 0)
         self.SetPyData(root_node.item, root_node)
-        self.skip_select = 0 # necessary to avoid an infinite loop on win32, as
-                             # SelectItem fires an EVT_TREE_SEL_CHANGED event
+        self.skip_select = 0  # necessary to avoid an infinite loop on
+                              # win32, as SelectItem fires an
+                              # EVT_TREE_SEL_CHANGED event
         self.title = ' '
         self.set_title(self.title)
-        
-        self.auto_expand = True # this control the automatic expansion of
+
+        self.auto_expand = True  # this control the automatic expansion of
                                 # nodes: it is set to False during xml loading
-        self._show_menu = misc.wxGladePopupMenu('widget') # popup menu to
+        self._show_menu = misc.wxGladePopupMenu('widget')  # popup menu to
                                                           # show toplevel
                                                           # widgets
-        SHOW_ID = wx.NewId()
-        self._show_menu.Append(SHOW_ID, _('Show'))
-        wx.EVT_TREE_SEL_CHANGED(self, id, self.on_change_selection)
-        wx.EVT_RIGHT_DOWN(self, self.popup_menu)
-        wx.EVT_LEFT_DCLICK(self, self.show_toplevel)
-        wx.EVT_MENU(self, SHOW_ID, self.show_toplevel)
+        self._show_menu.Append(wx.ID_ANY, _('Show'))
+        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.on_change_selection)
+        self.Bind(wx.EVT_RIGHT_DOWN, self.popup_menu)
+        self.Bind(wx.EVT_LEFT_DCLICK, self.show_toplevel)
+        self.Bind(wx.EVT_MENU, self.show_toplevel)
+
         def on_key_down(event):
             evt_flags = 0
-            if event.ControlDown(): evt_flags = wx.ACCEL_CTRL
+            if event.ControlDown():
+                evt_flags = wx.ACCEL_CTRL
             evt_key = event.GetKeyCode()
             for flags, key, function in misc.accel_table:
                 if evt_flags == flags and evt_key == key:
                     wx.CallAfter(function)
                     break
             event.Skip()
-        wx.EVT_KEY_DOWN(self, on_key_down)
+        self.Bind(wx.EVT_KEY_DOWN, on_key_down)
 
     def _build_label(self, node):
         s = node.widget.name
         if node.widget.klass != node.widget.base and \
-               node.widget.klass != 'wxScrolledWindow': # special case...
+               node.widget.klass != 'wxScrolledWindow':  # special case...
             s += ' (%s)' % node.widget.klass
         return s
-        
-    def add(self, child, parent=None, image=None): # is image still used?
+
+    def add(self, child, parent=None, image=None):  # is image still used?
         """\
         appends child to the list of parent's children
         """
         Tree.add(self, child, parent)
-        import common
         name = child.widget.__class__.__name__
         index = WidgetTree.images.get(name, -1)
         if parent is None: parent = parent.item = self.GetRootItem()
@@ -338,16 +467,13 @@ class WidgetTree(wx.TreeCtrl, Tree):
         if parent.children is None:
             self.add(child, parent, image)
             return
-        import common
         name = child.widget.__class__.__name__
         image_index = WidgetTree.images.get(name, -1)
-        if parent is None: parent = parent.item = self.GetRootItem()
+        if parent is None:
+            parent = parent.item = self.GetRootItem()
 
         index = 0
-        if misc.check_wx_version(2, 5):
-            item, cookie = self.GetFirstChild(parent.item)
-        else:
-            item, cookie = self.GetFirstChild(parent.item, 1)
+        item, cookie = self.GetFirstChild(parent.item)
         while item.IsOk():
             item_pos = self.GetPyData(item).widget.pos
             if pos < item_pos: break
@@ -366,7 +492,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.app.check_codegen(child.widget)
 
     def remove(self, node=None):
-        self.app.saved = False # update the status of the app
+        self.app.saved = False  # update the status of the app
         Tree.remove(self, node)
         if node is not None:
             try:
@@ -389,7 +515,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         app = self.GetPyData(self.GetRootItem())
         app.widget.show_properties()
 
-    def refresh_name(self, node, oldname=None): #, name=None):
+    def refresh_name(self, node, oldname=None):  # , name=None):
         if oldname is not None:
             try:
                 #del self.names[self.GetItemText(node.item)]
@@ -425,29 +551,20 @@ class WidgetTree(wx.TreeCtrl, Tree):
             except AttributeError:
                 pass
             except Exception:
-                import traceback
-                traceback.print_exc()
+                self._logger.exception(_('Internal Error'))
 
     def popup_menu(self, event):
         node = self._find_item_by_pos(*event.GetPosition())
-        if not node: return
-        else:
-            self.select_item(node)
-            item = node.widget
+        if not node:
+            return
+        self.select_item(node)
+        item = node.widget
         if not item.widget or not item.is_visible():
-            import edit_windows
-            #if isinstance(item, edit_windows.TopLevelBase):
             if node.parent is self.root:
                 self._show_menu.SetTitle(item.name)
                 self.PopupMenu(self._show_menu, event.GetPosition())
             return
-        try:
-            x, y = self.ClientToScreen(event.GetPosition())
-            x, y = item.widget.ScreenToClient((x, y))
-            event.m_x, event.m_y = x, y
-            item.popup_menu(event)
-        except AttributeError:
-            import traceback; traceback.print_exc()
+        item.popup_menu(event)
 
     def expand(self, node=None, yes=True):
         """\
@@ -460,8 +577,10 @@ class WidgetTree(wx.TreeCtrl, Tree):
     def set_title(self, value):
         if value is None: value = ""
         self.title = value
-        try: self.GetParent().SetTitle(_('wxGlade: Tree %s') % value)
-        except: pass
+        try:
+            self.GetParent().SetTitle(_('wxGlade: Tree %s') % value)
+        except:
+            pass
 
     def get_title(self):
         if not self.title: self.title = ' '
@@ -492,6 +611,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 wx.EndBusyCursor()
         else:
             import edit_sizers
+
             def show_rec(node):
                 node.widget.show_widget(True)
                 self.expand(node, True)
@@ -502,7 +622,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
 ##                 if isinstance(w, edit_sizers.SizerBase): return
 ##                 elif not w.properties['size'].is_active() and \
 ##                          w.sizer and w.sizer.toplevel:
-##                     w.sizer.fit_parent()                    
+##                     w.sizer.fit_parent()
             show_rec(node)
 
     def show_toplevel(self, event):
@@ -514,7 +634,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         except AttributeError:
             # if we are here, event is a CommandEvent and not a MouseEvent
             node = self.GetPyData(self.GetSelection())
-            self.expand(node) # if we are here, the widget must be shown
+            self.expand(node)  # if we are here, the widget must be shown
         else:
             node = self._find_item_by_pos(x, y, True)
         if node is not None:
@@ -550,7 +670,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         Tree.change_node(self, node, widget)
         self.SetItemImage(node.item, self.images.get(
             widget.__class__.__name__, -1))
-        self.SetItemText(node.item, self._build_label(node)) #widget.name)
+        self.SetItemText(node.item, self._build_label(node))  # widget.name)
 
     def change_node_pos(self, node, new_pos):
         if new_pos >= self.GetChildrenCount(node.parent.item, False):
@@ -560,7 +680,12 @@ class WidgetTree(wx.TreeCtrl, Tree):
         old_item = node.item
         image = self.GetItemImage(node.item)
         self.Freeze()
-        #print self._build_label(node), index, new_pos
+        #self._logger.debug(
+        #    '%s, %s, %s',
+        #    self._build_label(node),
+        #    index,
+        #    new_pos
+        #    )
         if index >= new_pos:
             node.item = self.InsertItemBefore(
                 node.parent.item, new_pos, self._build_label(node), image)
@@ -568,6 +693,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             node.item = self.InsertItemBefore(
                 node.parent.item, new_pos+1, self._build_label(node), image)
         self.SetPyData(node.item, node)
+
         def append(parent, node):
             idx = WidgetTree.images.get(node.widget.__class__.__name__, -1)
             node.item = self.AppendItem(parent.item,
@@ -630,10 +756,10 @@ class WidgetTree(wx.TreeCtrl, Tree):
         while item.Ok() and index < len(path):
             widget = self.GetPyData(item).widget
             name = path[index]
-            if index == 0 and type(name) == type(()):
+            if index == 0 and isinstance(name, types.TupleType):
                 name, pos = name
             if misc.streq(widget.name, name):
-                #print 'OK:', widget.name
+                #self._logger.debug('OK: %s', widget.name)
                 #self.EnsureVisible(item)
                 itemok = item
                 if parent is None:
@@ -642,7 +768,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 item, cookie = self._get_first_child(item)
                 index += 1
             else:
-                #print 'NO:', widget.name
+                #self._logger.debug('NO: %s', widget.name)
                 item = self.GetNextSibling(item)
         if itemok is not None:
             node = self.GetPyData(itemok)
@@ -651,12 +777,9 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 if pos is not None:
                     misc.get_toplevel_parent(parent.widget).SetPosition(pos)
             self.select_item(node)
-        
+
     def _get_first_child(self, item):
-        if misc.check_wx_version(2, 5):
-            return self.GetFirstChild(item)
-        else:
-            return self.GetFirstChild(item, 1)
-        
+        return self.GetFirstChild(item)
+
 # end of class WidgetTree
 
