@@ -1,70 +1,53 @@
-# combo_box.py: wxComboBox objects
-# $Id: combo_box.py,v 1.28 2007/03/27 07:02:02 agriggio Exp $
-#
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""\
+wxComboBox objects
+
+@copyright: 2002-2007 Alberto Griggio
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
 import wx
-import common, misc
-from edit_windows import ManagedBase
+
+import common
+import compat
+import config
+import misc
+from edit_windows import ManagedBase, EditStylesMixin
 from tree import Tree
 from widget_properties import *
-
 from ChoicesProperty import *
 
-if wx.Platform == '__WXMSW__':
-    # why on Windows combo boxes give segfaults? Need to investigate, but
-    # for now replace them with choices
-    # this seems to be because of the style of wxPanel: if there's a
-    # wxTAB_TRAVERSAL, we have troubles -- now it should be fixed...
-    class wxComboBox2(wx.ComboBox):
-        # on windows GetBestSize considers also the drop down menu, while we
-        # don't want it to be included
-        def GetBestSize(self):
-            w, h = wx.ComboBox.GetBestSize(self)
-            n = self.GetCount()
-            return w, h/(n+1)
 
-        def GetSize(self):
-            return self.GetClientSize()
-else:
-    wxComboBox2 = wx.ComboBox
+class EditComboBox(ManagedBase, EditStylesMixin):
+    """\
+    Class to handle wxComboBox objects
+    """
 
+    update_widget_style = False
 
-class EditComboBox(ManagedBase):
-
-    events = ['EVT_COMBOBOX', 'EVT_TEXT', 'EVT_TEXT_ENTER']
-    
     def __init__(self, name, parent, id, choices, sizer, pos, property_window,
                  show=True):
-        """\
-        Class to handle wxComboBox objects
-        """
-        import config
+
+        # Initialise parent classes
         ManagedBase.__init__(self, name, 'wxComboBox', parent, id, sizer,
                              pos, property_window, show=show)
+        EditStylesMixin.__init__(self)
+
+        # initialise instance variables
         self.choices = choices
-        if len(choices): self.selection = 0
-        else: self.selection = -1
-        self.style = 0
-        # properties
+        if len(choices):
+            self.selection = 0
+        else:
+            self.selection = -1
+        if config.preferences.default_border:
+            self.border = config.preferences.default_border_size
+            self.flag = wx.ALL
+
+        # initialise properties remaining staff
         self.access_functions['choices'] = (self.get_choices, self.set_choices)
         self.access_functions['style'] = (self.get_style, self.set_style)
-        style_labels = ('#section#' + _('Style'), 'wxCB_SIMPLE','wxCB_DROPDOWN',
-                        'wxCB_READONLY', 'wxCB_SORT')
-        self.style_pos = [ eval('wx.' + s[2:]) for s in style_labels[1:] ]
-	self.tooltips = (_("Creates a combobox with a permanently displayed list."
-                         " Windows only."),
-                         _("Creates a combobox with a drop-down list."),
-                         _("Same as wxCB_DROPDOWN but only the strings specified "
-                         "as the combobox choices can be selected, it is "
-                         "impossible to select (even from a program) a string "
-                         "which is not in the choices list."),
-                         _("Sorts the entries in the list alphabetically."))
-        self.properties['style'] = CheckListProperty(self, 'style', None,
-                                                     style_labels,
-                                                     tooltips=self.tooltips)
+        self.properties['style'] = CheckListProperty(
+            self, 'style', self.widget_writer)
         self.properties['choices'] = ChoicesProperty(self, 'choices', None,
                                                      [('Label',
                                                        GridProperty.STRING)],
@@ -74,13 +57,9 @@ class EditComboBox(ManagedBase):
         self.choices = list(choices)
         self.properties['selection'] = SpinProperty(self, 'selection', None,
                                                     r=(0, len(choices)-1), label=_("selection"))
-        # 2003-09-04 added default_border
-        if config.preferences.default_border:
-            self.border = config.preferences.default_border_size
-            self.flag = wx.ALL
 
     def create_widget(self):
-        self.widget = wxComboBox2(self.parent.widget, self.id,
+        self.widget = wx.ComboBox(self.parent.widget, self.id,
                                  choices=self.choices)
         self.set_selection(self.selection)
         wx.EVT_LEFT_DOWN(self.widget, self.on_set_focus)
@@ -96,7 +75,7 @@ class EditComboBox(ManagedBase):
         szr.Add(self.properties['selection'].panel, 0, wx.EXPAND)
         szr.Add(self.properties['choices'].panel, 1, wx.EXPAND)
         panel.SetAutoLayout(True)
-        panel.SetSizer(szr)
+        compat.SizerItem_SetSizer(panel, szr)
         szr.Fit(panel)
         self.notebook.AddPage(panel, 'Widget')
         self.properties['choices'].set_col_sizes([-1])
@@ -119,33 +98,11 @@ class EditComboBox(ManagedBase):
         self.properties['selection'].set_range(0, len(self.choices)-1)
         if self.widget:
             self.widget.Clear()
-            for c in self.choices: self.widget.Append(c)
+            for c in self.choices:
+                self.widget.Append(c)
             if not self.properties['size'].is_active():
                 self.sizer.set_item(self.pos, size=self.widget.GetBestSize())
 
-    def get_style(self):
-        retval = [0] * len(self.style_pos)
-        try:
-            for i in range(len(self.style_pos)):
-                if self.style & self.style_pos[i]:
-                    retval[i] = 1
-        except AttributeError: pass
-        return retval
-
-    def set_style(self, value):
-        value = self.properties['style'].prepare_value(value)
-        self.style = 0
-        for v in range(len(value)):
-            if value[v]:
-                self.style |= self.style_pos[v]
-##         if self.widget:
-##             self.SetWindowStyleFlag(style)
-
-    def create_widget(self):
-        self.widget = wxComboBox2(self.parent.widget, self.id,
-                                  choices=self.choices)
-        wx.EVT_LEFT_DOWN(self.widget, self.on_set_focus)
-
     def get_property_handler(self, prop_name):
         if prop_name == 'choices':
             return ChoicesHandler(self)
@@ -153,7 +110,7 @@ class EditComboBox(ManagedBase):
 
 # end of class EditComboBox
 
-        
+
 def builder(parent, sizer, pos, number=[1]):
     """\
     factory function for EditComboBox objects.
@@ -162,35 +119,38 @@ def builder(parent, sizer, pos, number=[1]):
     while common.app_tree.has_name(name):
         number[0] += 1
         name = 'combo_box_%d' % number[0]
-    choice = EditComboBox(name, parent, wx.NewId(), #[misc._encode('choice 1')],
+    choice = EditComboBox(name, parent, wx.NewId(),
                           [], sizer, pos, common.property_panel)
     node = Tree.Node(choice)
 #    sizer.set_item(pos, size=choice.GetBestSize())
     choice.node = node
     choice.show_widget(True)
-    common.app_tree.insert(node, sizer.node, pos-1)
+    common.app_tree.insert(node, sizer.node, pos - 1)
+
 
 def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """\
-    factory to build EditComboBox objects from an xml file
+    factory to build EditComboBox objects from a XML file
     """
     from xml_parse import XmlParsingError
-    try: name = attrs['name']
-    except KeyError: raise XmlParsingError, _("'name' attribute missing")
+    try:
+        name = attrs['name']
+    except KeyError:
+        raise XmlParsingError(_("'name' attribute missing"))
     if sizer is None or sizeritem is None:
-        raise XmlParsingError, _("sizer or sizeritem object cannot be None")
+        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
     choice = EditComboBox(name, parent, wx.NewId(), [], sizer, pos,
                           common.property_panel)
     sizer.set_item(choice.pos, option=sizeritem.option,
                    flag=sizeritem.flag, border=sizeritem.border)
-##                    size=choice.GetBestSize())
     node = Tree.Node(choice)
     choice.node = node
-    if pos is None: common.app_tree.add(node, sizer.node)
-    else: common.app_tree.insert(node, sizer.node, pos-1)
+    if pos is None:
+        common.app_tree.add(node, sizer.node)
+    else:
+        common.app_tree.insert(node, sizer.node, pos - 1)
     return choice
 
-    
 
 def initialize():
     """\
@@ -200,4 +160,4 @@ def initialize():
     common.widgets['EditComboBox'] = builder
     common.widgets_from_xml['EditComboBox'] = xml_builder
 
-    return common.make_object_button('EditComboBox', 'icons/combo_box.xpm')
+    return common.make_object_button('EditComboBox', 'combo_box.xpm')
