@@ -1,33 +1,31 @@
-# Grid.py: wxGrid objects
-# $Id: grid.py,v 1.33 2007/03/27 07:01:58 agriggio Exp $
-#
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""\
+wxGrid objects
+
+@copyright: 2002-2007 Alberto Griggio
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
 import wx
 from wx.grid import *
-import common, misc
+import common
+import compat
+import misc
 from edit_windows import ManagedBase
 from tree import Tree
 from widget_properties import *
+from wcodegen.taghandler import BaseXmlBuilderTagHandler
+
 
 class GridColsProperty(GridProperty):
     def write(self, outfile, tabs):
-        from xml.sax.saxutils import escape, quoteattr
-        write = outfile.write
-        write('    ' * tabs + '<columns>\n')
-        tab_s = '    ' * (tabs+1)
-        import widget_properties
-        value = self.get_value() # this is a list
-        for i in range(len(value)):
-            val = value[i] # this is another list
-            _label = escape(widget_properties._encode(val[0]))
-            _size  = escape(widget_properties._encode(val[1]))
-            write('%s<column size=%s>%s</column>\n' % (tab_s,
-                                                       quoteattr(_size),
-                                                       _label ))
-        write('    ' * tabs + '</columns>\n')
+        value = self.get_value()
+        inner_xml = u''
+        for label, size in value:
+            inner_xml += common.format_xml_tag(
+                u'column', label, tabs + 1, size=size)
+        stmt = common.format_xml_tag(u'columns', inner_xml, tabs, is_xml=True)
+        outfile.write(stmt)
 
     def _get_label(self, col):
         s = []
@@ -53,8 +51,10 @@ class GridColsProperty(GridProperty):
 # end of class GridColumnsProperty
 
 
-class ColsHandler:
+class ColsHandler(BaseXmlBuilderTagHandler):
+
     def __init__(self, parent):
+        super(ColsHandler, self).__init__()
         self.parent = parent
         self.columns = []
         self.curr_col = []
@@ -70,37 +70,15 @@ class ColsHandler:
             self.parent.properties['columns'].set_value(self.columns)
             return True
         elif name == 'column':
-            self.columns.append(["".join(self.curr_col), self.curr_size])
-            self.curr_col = []
+            char_data = self.get_char_data()
+            self.columns.append([char_data, self.curr_size])
         return False
 
-    def char_data(self, data):
-        self.curr_col.append(data)
-
 # end of class ColsHandler
 
 
 class EditGrid(ManagedBase):
 
-    events = [
-        'EVT_GRID_CMD_CELL_LEFT_CLICK',
-        'EVT_GRID_CMD_CELL_RIGHT_CLICK',
-        'EVT_GRID_CMD_CELL_LEFT_DCLICK',
-        'EVT_GRID_CMD_CELL_RIGHT_DCLICK',
-        'EVT_GRID_CMD_LABEL_LEFT_CLICK',
-        'EVT_GRID_CMD_LABEL_RIGHT_CLICK',
-        'EVT_GRID_CMD_LABEL_LEFT_DCLICK',
-        'EVT_GRID_CMD_LABEL_RIGHT_DCLICK',
-        'EVT_GRID_CMD_CELL_CHANGE',
-        'EVT_GRID_CMD_SELECT_CELL',
-        'EVT_GRID_CMD_EDITOR_HIDDEN',
-        'EVT_GRID_CMD_EDITOR_SHOWN',
-        'EVT_GRID_CMD_COL_SIZE',
-        'EVT_GRID_CMD_ROW_SIZE',
-        'EVT_GRID_CMD_RANGE_SELECT',
-        'EVT_GRID_CMD_EDITOR_CREATED',
-        ]
-    
     def __init__(self, name, parent, id, sizer, pos, property_window,
                  show=True):
         """\
@@ -117,75 +95,78 @@ class EditGrid(ManagedBase):
         self.enable_grid_resize = True
         self.lines_color = '#000000'
         self.label_bg_color = '#C0C0C0'
-        self.selection_mode = 0 # == wxGrid.wxGridSelectCells
+        self.selection_mode = 0  # == wxGrid.wxGridSelectCells
         self.create_grid = True
-        self.columns = [ ['A','-1'] , ['B','-1'] , ['C','-1'] ]
-        
+        self.columns = [['A', '-1'], ['B', '-1'], ['C', '-1']]
+
         ManagedBase.__init__(self, name, 'wxGrid', parent, id, sizer, pos,
                              property_window, show=show)
         props = self.properties
         af = self.access_functions
         af['create_grid'] = (self.get_create_grid, self.set_create_grid)
-        props['create_grid'] = CheckBoxProperty(self, 'create_grid', None,
-                                                write_always=True,
-                                                label=_("create_grid"))
+        props['create_grid'] = CheckBoxProperty(
+            self, 'create_grid', write_always=True, label=_("Create grid"))
         af['row_label_size'] = (self.get_row_label_size,
                                 self.set_row_label_size)
-        props['row_label_size'] = SpinProperty(self, 'row_label_size',
-                                               None, can_disable=True,
-                                               label=_("row_label_size"))
+        props['row_label_size'] = SpinProperty(
+            self, 'row_label_size', None, can_disable=True,
+            label=_("Row label size"), omitter='create_grid')
         af['col_label_size'] = (self.get_col_label_size,
                                 self.set_col_label_size)
-        props['col_label_size'] = SpinProperty(self, 'col_label_size',
-                                               None, can_disable=True,
-                                               label=_("col_label_size"))
+        props['col_label_size'] = SpinProperty(
+            self, 'col_label_size', None, can_disable=True,
+            label=_("Column label size"), omitter='create_grid')
         af['enable_editing'] = (self.get_enable_editing,
                                 self.set_enable_editing)
-        props['enable_editing'] = CheckBoxProperty(self, 'enable_editing',
-                                                   None, write_always=True,
-                                                   label=_("enable_editing"))
+        props['enable_editing'] = CheckBoxProperty(
+            self, 'enable_editing', None, write_always=True,
+            label=_("Enable editing"), omitter='create_grid')
         af['enable_grid_lines'] = (self.get_enable_grid_lines,
                                    self.set_enable_grid_lines)
         props['enable_grid_lines']= CheckBoxProperty(
             self, 'enable_grid_lines', None, write_always=True,
-            label=_("enable_grid_lines"))
+            label=_("Enable grid lines"), omitter='create_grid')
         af['rows_number'] = (self.get_rows_number, self.set_rows_number)
-        props['rows_number'] = SpinProperty(self, 'rows_number', None,
-                                            label=_("rows_number"))
+        props['rows_number'] = SpinProperty(
+            self, 'rows_number', None,
+            label=_("Number of rows"), omitter='create_grid')
         af['enable_col_resize'] = (self.get_enable_col_resize,
                                    self.set_enable_col_resize)
         props['enable_col_resize']= CheckBoxProperty(
             self, 'enable_col_resize', None, write_always=True,
-            label=_("enable_col_resize"))
+            label=_("Enable resizing columns"), omitter='create_grid')
         af['enable_row_resize'] = (self.get_enable_row_resize,
                                    self.set_enable_row_resize)
         props['enable_row_resize'] = CheckBoxProperty(
-            self, 'enable_row_resize',
-            None, write_always=True, label=_("enable_row_resize"))
+            self, 'enable_row_resize', None, write_always=True,
+            label=_("Enable resizing rows"), omitter='create_grid')
         af['enable_grid_resize'] = (self.get_enable_grid_resize,
                                     self.set_enable_grid_resize)
         props['enable_grid_resize'] = CheckBoxProperty(
-            self, 'enable_grid_resize',
-            None, write_always=True, label=_("enable_grid_resize"))
+            self, 'enable_grid_resize', None, write_always=True,
+            label=_("Enable resizing grid"), omitter='create_grid')
         af['lines_color'] = (self.get_lines_color, self.set_lines_color)
         props['lines_color']= ColorDialogProperty(
-            self, 'lines_color', None, label=_("lines_color"))
+            self, 'lines_color', None,
+            label=_("Lines colour"), omitter='create_grid')
         af['label_bg_color'] = (self.get_label_bg_color,
                                 self.set_label_bg_color)
         props['label_bg_color']= ColorDialogProperty(
-            self, 'label_bg_color', None, label=_("label_bg_color"))
+            self, 'label_bg_color', None,
+            label=_("Label background colour"), omitter='create_grid')
         af['selection_mode'] = (self.get_selection_mode,
                                 self.set_selection_mode)
-        props['selection_mode'] = RadioProperty(self, 'selection_mode', None, 
-                                                ['wxGrid.wxGridSelectCells',
-                                                 'wxGrid.wxGridSelectRows',
-                                                 'wxGrid.wxGridSelectColumns'],
-                                                label=_("selection_mode"))
+        props['selection_mode'] = RadioProperty(
+            self, 'selection_mode', None,
+            ['wxGrid.wxGridSelectCells',
+             'wxGrid.wxGridSelectRows',
+             'wxGrid.wxGridSelectColumns'],
+            label=_("Selection mode"), omitter='create_grid')
         af['columns'] = (self.get_columns, self.set_columns)
-        props['columns'] = GridColsProperty(self, 'columns', None, 
-                                            [ ('Label', GridProperty.STRING),
-                                              ('Size', GridProperty.INT) ],
-                                            label=_("columns"))
+        props['columns'] = GridColsProperty(
+            self, 'columns', None,
+            [('Label', GridProperty.STRING), ('Size', GridProperty.INT)],
+            label=_("Columns"), omitter='create_grid')
 
     def create_properties(self):
         ManagedBase.create_properties(self)
@@ -203,6 +184,7 @@ class EditGrid(ManagedBase):
         self.properties['lines_color'].display(panel)
         self.properties['label_bg_color'].display(panel)
         self.properties['selection_mode'].display(panel)
+        self.toggle_blocked_properties()
         szr = wx.BoxSizer(wx.VERTICAL)
         szr.Add(self.properties['create_grid'].panel, 0, wx.EXPAND)
         szr.Add(wx.StaticLine(panel, -1), 0, wx.ALL|wx.EXPAND, 5)
@@ -224,7 +206,7 @@ class EditGrid(ManagedBase):
         szr.Add(self.properties['enable_grid_resize'].panel, 0, wx.EXPAND)
         szr.Add(self.properties['selection_mode'].panel, 0, wx.ALL|wx.EXPAND, 5)
         panel.SetAutoLayout(1)
-        panel.SetSizer(szr)
+        compat.SizerItem_SetSizer(panel, szr)
         szr.Fit(panel)
         w, h = panel.GetClientSize()
         self.notebook.AddPage(panel, _('Widget'))
@@ -233,7 +215,7 @@ class EditGrid(ManagedBase):
         self.properties['columns'].set_col_sizes([-1, 0])
 
     def create_widget(self):
-        self.widget = Grid(self.parent.widget, self.id,(200,200))
+        self.widget = Grid(self.parent.widget, self.id, (200, 200))
         self.widget.CreateGrid(self.rows_number, len(self.columns))
         if not self.properties['label_bg_color'].is_active():
             self.label_bg_color = misc.color_to_string(
@@ -254,27 +236,38 @@ class EditGrid(ManagedBase):
         self.widget.SetLabelBackgroundColour(misc.string_to_color(
             self.label_bg_color))
         i = 0
-        for l, s in self.columns:
-            try: s1 = int(s)
-            except: s1 = 0
-            self.widget.SetColLabelValue(i, l)
-            if s1 > 0:
-                self.widget.SetColSize(i, s1)
+        for label, size_str in self.columns:
+            try:
+                size_int = int(size_str)
+            except:
+                size_int = 0
+            label = label.replace('\\n', '\n')
+            self.widget.SetColLabelValue(i, label)
+            if size_int > 0:
+                self.widget.SetColSize(i, size_int)
             i += 1
 
         self.set_selection_mode(self.selection_mode)
         # following two events are to permit select grid from designer frame
-        EVT_GRID_CELL_LEFT_CLICK(self.widget, self.on_set_focus)  
+        EVT_GRID_CELL_LEFT_CLICK(self.widget, self.on_set_focus)
         EVT_GRID_LABEL_LEFT_CLICK(self.widget, self.on_set_focus)
         # these are to show the popup menu on right click
         EVT_GRID_CELL_RIGHT_CLICK(self.widget, self.popup_menu)
         EVT_GRID_LABEL_RIGHT_CLICK(self.widget, self.popup_menu)
 
+    def toggle_blocked_properties(self):
+        to_block = not self.get_create_grid()
+        for name in self.get_property_blocking('create_grid'):
+            widget = self.properties[name]
+            if getattr(widget, 'toggle_blocked', None):
+                widget.toggle_blocked(to_block)
+
     def get_create_grid(self):
         return self.create_grid
 
     def set_create_grid(self, value):
         self.create_grid = bool(int(value))
+        self.toggle_blocked_properties()
 
     def get_row_label_size(self):
         return self.row_label_size
@@ -371,9 +364,6 @@ class EditGrid(ManagedBase):
 
     def get_selection_mode(self):
         return self.selection_mode
-##         if self.selection_mode == wxGrid.wxGridSelectCells:   return 0
-##         if self.selection_mode == wxGrid.wxGridSelectRows:    return 1
-##         if self.selection_mode == wxGrid.wxGridSelectColumns: return 2
 
     def set_selection_mode(self, value):
         _sel_modes = {
@@ -384,23 +374,19 @@ class EditGrid(ManagedBase):
         if value in _sel_modes:
             self.selection_mode = _sel_modes[value]
         else:
-            try: value = int(value)
-            except: pass
-            else: self.selection_mode = value
-##         if value == 0:
-##             self.selection_mode = wxGrid.wxGridSelectCells
-##         elif value == 1:
-##             self.selection_mode = wxGrid.wxGridSelectRows
-##         else:
-##             self.selection_mode = wxGrid.wxGridSelectColumns
-        # no operation on the grid.
+            try:
+                value = int(value)
+            except:
+                pass
+            else:
+                self.selection_mode = value
 
     def get_columns(self):
         return self.columns
 
     def set_columns(self, cols):
         # first of all, adjust col number
-        _oldcolnum = len(self.columns) 
+        _oldcolnum = len(self.columns)
         _colnum = len(cols)
         self.columns = cols
         if not self.widget: return
@@ -409,21 +395,24 @@ class EditGrid(ManagedBase):
         if _colnum < _oldcolnum:
             self.widget.DeleteCols(0, _oldcolnum - _colnum)
         i = 0
-        for l, s in cols:
-            try: s1 = int(s)
-            except: s1 = 0
-            self.widget.SetColLabelValue(i, misc.wxstr(l))
-            if s1 > 0:
-                self.widget.SetColSize(i, s1)
+        for label, size_str in cols:
+            try:
+                size_int = int(size_str)
+            except:
+                size_int = 0
+            label = label.replace('\\n', '\n')
+            self.widget.SetColLabelValue(i, misc.wxstr(label))
+            if size_int > 0:
+                self.widget.SetColSize(i, size_int)
             i += 1
         self.widget.ForceRefresh()
-            
+
     def get_property_handler(self, name):
         if name == 'columns': return ColsHandler(self)
         return ManagedBase.get_property_handler(self, name)
 
 # end of class EditGrid
-        
+
 
 def builder(parent, sizer, pos, number=[1]):
     """\
@@ -437,31 +426,35 @@ def builder(parent, sizer, pos, number=[1]):
                     common.property_panel)
     # A grid should be wx.EXPANDed and 'option' should be 1,
     # or you can't see it.
-    grid.set_option(1)  
-    grid.set_flag("wxEXPAND")
+    grid.set_option(1)
+    grid.esm_border.set_style("wxEXPAND")
     node = Tree.Node(grid)
     grid.node = node
     grid.show_widget(True)
-    common.app_tree.insert(node, sizer.node, pos-1)
+    common.app_tree.insert(node, sizer.node, pos - 1)
 
 
 def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """\
-    factory to build EditGrid objects from an xml file
+    factory to build EditGrid objects from a XML file
     """
     from xml_parse import XmlParsingError
-    try: label = attrs['name']
-    except KeyError: raise XmlParsingError, _("'name' attribute missing")
+    try:
+        label = attrs['name']
+    except KeyError:
+        raise XmlParsingError(_("'name' attribute missing"))
     if sizer is None or sizeritem is None:
-        raise XmlParsingError, _("sizer or sizeritem object cannot be None")
+        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
     grid = EditGrid(label, parent, wx.NewId(), sizer,
                     pos, common.property_panel, show=False)
     sizer.set_item(grid.pos, option=sizeritem.option, flag=sizeritem.flag,
-                   border=sizeritem.border) #, size=(100,100))  #HELP#
+                   border=sizeritem.border)
     node = Tree.Node(grid)
     grid.node = node
-    if pos is None: common.app_tree.add(node, sizer.node)
-    else: common.app_tree.insert(node, sizer.node, pos-1)
+    if pos is None:
+        common.app_tree.add(node, sizer.node)
+    else:
+        common.app_tree.insert(node, sizer.node, pos - 1)
     return grid
 
 
@@ -473,5 +466,5 @@ def initialize():
     common.widgets['EditGrid'] = builder
     common.widgets_from_xml['EditGrid'] = xml_builder
 
-    return common.make_object_button('EditGrid', 'icons/grid.xpm')
+    return common.make_object_button('EditGrid', 'grid.xpm')
 
