@@ -1,67 +1,82 @@
-# codegen.py: code generator functions for wxPanel objects
-# $Id: codegen.py,v 1.17 2005/11/20 10:50:48 agriggio Exp $
-#
-# Copyright (c) 2002-2005 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""\
+Code generator functions for wxPanel objects
+
+@copyright: 2002-2007 Alberto Griggio
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
 
 import common
+import wcodegen
 
 
-class PythonCodeGenerator:
+class PythonPanelGenerator(wcodegen.PythonWidgetCodeWriter):
     def get_code(self, panel):
-        pygen = common.code_writers['python']
-        cn = pygen.cn
-        cn_f = pygen.cn_f
         prop = panel.properties
-        try: scrollable = int(prop['scrollable'])
-        except: scrollable = False
-        id_name, id = pygen.generate_code_id(panel)
-        if not panel.parent.is_toplevel: parent = 'self.%s' % panel.parent.name
-        else: parent = 'self'
+        try:
+            scrollable = int(prop['scrollable'])
+        except:
+            scrollable = False
+        id_name, id = self.codegen.generate_code_id(panel)
+        parent = self.format_widget_access(panel.parent)
         if panel.is_toplevel:
             l = []
             if id_name: l.append(id_name)
             l.append('self.%s = %s(%s, %s)\n' %
-                     (panel.name, panel.klass, parent, id))
+                     (panel.name, self.codegen.without_package(panel.klass),
+                      parent, id))
             return l, [], []
         init = []
-        if id_name: init.append(id_name)
+        if id_name:
+            init.append(id_name)
         style = prop.get("style", 'wxTAB_TRAVERSAL')
         if scrollable or style != 'wxTAB_TRAVERSAL':
-            style = ", style=%s" % cn_f(style)
-        else: style = ''
-        # ALB 2005-11-19
+            style = ", style=%s" % self.cn_f(style)
+        else:
+            style = ''
         if not int(panel.properties.get('no_custom_class', False)) \
                or panel.preview:
-            if scrollable: klass = cn('wxScrolledWindow')
-            else: klass = cn('wxPanel')
+            if scrollable:
+                klass = self.cn('wxScrolledWindow')
+            else:
+                klass = self.cn('wxPanel')
         else:
             klass = panel.klass
+            if klass in ('wxPanel', 'wxScrolledWindow'):
+                klass = self.cn(klass)
         init.append('self.%s = %s(%s, %s%s)\n' % \
                     (panel.name, klass, parent, id, style))
-        props_buf = pygen.generate_common_properties(panel)
+        props_buf = self.codegen.generate_common_properties(panel)
         if scrollable:
             sr = prop.get('scroll_rate', '0, 0')
             props_buf.append('self.%s.SetScrollRate(%s)\n' % (panel.name, sr))
         return init, props_buf, []
 
     def get_properties_code(self, obj):
-        pygen = common.code_writers['python']
-        prop = obj.properties
-        try: scrollable = int(prop['scrollable'])
-        except: scrollable = False
-        props_buf = pygen.generate_common_properties(obj)
+        try:
+            scrollable = int(obj.properties['scrollable'])
+        except:
+            scrollable = False
+        props_buf = self.codegen.generate_common_properties(obj)
         if scrollable:
-            sr = prop.get('scroll_rate', '0, 0')
+            sr = obj.properties.get('scroll_rate', '0, 0')
             props_buf.append('self.SetScrollRate(%s)\n' % sr)
         return props_buf
 
-# end of class PythonCodeGenerator
+    def get_layout_code(self, obj):
+        ret = ['self.Layout()\n']
+        try:
+            if int(obj.properties['centered']):
+                ret.append('self.Centre()\n')
+        except (KeyError, ValueError):
+            pass
+        return ret
+
+# end of class PythonPanelGenerator
 
 
-class CppCodeGenerator:
-    constructor = [('wxWindow*', 'parent'), ('int', 'id'),
+class CppPanelGenerator(wcodegen.CppWidgetCodeWriter):
+    constructor = [('wxWindow*', 'parent'), ('wxWindowID', 'id'),
                    ('const wxPoint&', 'pos', 'wxDefaultPosition'),
                    ('const wxSize&', 'size', 'wxDefaultSize'),
                    ('long', 'style', '0')]
@@ -70,15 +85,20 @@ class CppCodeGenerator:
         """\
         generates the C++ code for wxPanel objects
         """
-        cppgen = common.code_writers['C++']
         prop = panel.properties
-        try: scrollable = int(prop['scrollable'])
-        except: scrollable = False
-        id_name, id = cppgen.generate_code_id(panel)
-        if id_name: ids = [ id_name ]
-        else: ids = []
-        if not panel.parent.is_toplevel: parent = '%s' % panel.parent.name
-        else: parent = 'this'
+        try:
+            scrollable = int(prop['scrollable'])
+        except:
+            scrollable = False
+        id_name, id = self.codegen.generate_code_id(panel)
+        if id_name:
+            ids = [id_name]
+        else:
+            ids = []
+        if not panel.parent.is_toplevel:
+            parent = '%s' % panel.parent.name
+        else:
+            parent = 'this'
         if panel.is_toplevel:
             l = ['%s = new %s(%s, %s);\n' %
                  (panel.name, panel.klass, parent, id)]
@@ -87,32 +107,33 @@ class CppCodeGenerator:
         style = prop.get("style", 'wxTAB_TRAVERSAL')
         if scrollable or style != 'wxTAB_TRAVERSAL':
             extra = ', wxDefaultPosition, wxDefaultSize, %s' % style
-        # ALB 2005-11-19
         if not int(panel.properties.get('no_custom_class', False)):
-            if scrollable: klass = 'wxScrolledWindow'
-            else: klass = 'wxPanel'
+            if scrollable:
+                klass = 'wxScrolledWindow'
+            else:
+                klass = 'wxPanel'
         else:
             klass = panel.klass
         init = ['%s = new %s(%s, %s%s);\n' %
                 (panel.name, klass, parent, id, extra) ]
-        props_buf = cppgen.generate_common_properties(panel)
+        props_buf = self.codegen.generate_common_properties(panel)
         if scrollable:
             sr = prop.get('scroll_rate', '0, 0')
             props_buf.append('%s->SetScrollRate(%s);\n' % (panel.name, sr))
         return init, ids, props_buf, []
 
     def get_properties_code(self, obj):
-        cppgen = common.code_writers['C++']
-        prop = obj.properties
-        try: scrollable = int(prop['scrollable'])
-        except: scrollable = False
-        props_buf = cppgen.generate_common_properties(obj)
+        try:
+            scrollable = int(obj.properties['scrollable'])
+        except:
+            scrollable = False
+        props_buf = self.codegen.generate_common_properties(obj)
         if scrollable:
-            sr = prop.get('scroll_rate', '0, 0')
+            sr = obj.properties.get('scroll_rate', '0, 0')
             props_buf.append('SetScrollRate(%s);\n' % sr)
         return props_buf
 
-# end of class CppCodeGenerator
+# end of class CppPanelGenerator
 
 
 def xrc_code_generator(obj):
@@ -128,14 +149,17 @@ def xrc_code_generator(obj):
             for prop in ('scrollable', 'scroll_rate'):
                 try: del self.properties[prop]
                 except KeyError: pass
+            if 'no_custom_class' in self.properties:
+                del self.properties['no_custom_class']
             xrcgen.DefaultXrcObject.write(self, *args, **kwds)
 
     return XrcCodeGenerator(obj)
 
 
 def initialize():
-    common.class_names['EditPanel'] = 'wxPanel'
-    common.class_names['EditTopLevelPanel'] = 'wxPanel'
+    klass = 'wxPanel'
+    common.class_names['EditPanel'] = klass
+    common.class_names['EditTopLevelPanel'] = klass
     common.toplevels['EditPanel'] = 1
     common.toplevels['EditTopLevelPanel'] = 1
 
@@ -144,16 +168,9 @@ def initialize():
     common.toplevels['EditScrolledWindow'] = 1
     common.toplevels['EditTopLevelScrolledWindow'] = 1
 
-    # python code generation functions
-    pygen = common.code_writers.get('python')
-    if pygen:
-        pygen.add_widget_handler('wxPanel', PythonCodeGenerator())
-        pygen.add_widget_handler('wxScrolledWindow', PythonCodeGenerator())
-    cppgen = common.code_writers.get('C++')
-    if cppgen:
-        cppgen.add_widget_handler('wxPanel', CppCodeGenerator())
-        cppgen.add_widget_handler('wxScrolledWindow', CppCodeGenerator())
-    xrcgen = common.code_writers.get('XRC')
-    if xrcgen:
-        xrcgen.add_widget_handler('wxPanel', xrc_code_generator)
-        xrcgen.add_widget_handler('wxScrolledWindow', xrc_code_generator)
+    common.register('python', klass, PythonPanelGenerator(klass))
+    common.register('python', 'wxScrolledWindow', PythonPanelGenerator(klass))
+    common.register('C++', klass, CppPanelGenerator(klass))
+    common.register('C++', 'wxScrolledWindow', CppPanelGenerator(klass))
+    common.register('XRC', klass, xrc_code_generator)
+    common.register('XRC', 'wxScrolledWindow', xrc_code_generator)
