@@ -1,19 +1,28 @@
-# toolbar.py: wxToolBar objects
-# $Id: toolbar.py,v 1.26 2007/08/07 12:18:33 agriggio Exp $
-#
-# Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
-# License: MIT (see license.txt)
-# THIS PROGRAM COMES WITH NO WARRANTY
+"""\
+wxToolBar objects
 
+@copyright: 2002-2007 Alberto Griggio
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+"""
+
+import re
+import StringIO
 import wx
-#from wxPython.lib.filebrowsebutton import FileBrowseButton
 from wx.lib.filebrowsebutton import FileBrowseButton
 
-import common, math, misc, os
+import common
+import compat
+import config
+import math
+import misc
+import os
 from tree import Tree
 from tool import *
 from widget_properties import *
-from edit_windows import EditBase, TopLevelBase, PreviewMixin
+from edit_windows import EditBase, PreviewMixin, EditStylesMixin
+from gui_mixins import BitmapMixin
+from wcodegen.taghandler import BaseXmlBuilderTagHandler
 
 
 class _MyBrowseButton(FileBrowseButton):
@@ -23,12 +32,11 @@ class _MyBrowseButton(FileBrowseButton):
         button =wx.Button(self, ID, misc.wxstr(self.buttonText))
         button.SetToolTipString(misc.wxstr(self.toolTip))
         w = button.GetTextExtent(self.buttonText)[0] + 10
-        if not misc.check_wx_version(2, 5, 2): button.SetSize((w, -1))
-        else: button.SetMinSize((w, -1))
+        button.SetMinSize((w, -1))
         wx.EVT_BUTTON(button, ID, self.OnBrowse)
         return button
 
-    def OnBrowse (self, event=None):
+    def OnBrowse(self, event=None):
         """ Going to browse for file... """
         current = self.GetValue()
         directory = os.path.split(current)
@@ -40,8 +48,8 @@ class _MyBrowseButton(FileBrowseButton):
             directory = directory [0]
         else:
             directory = self.startDirectory
-        value = misc.FileSelector(self.dialogTitle, directory, current,
-                                  wildcard=self.fileMask, flags=self.fileMode)
+        value = wx.FileSelector(self.dialogTitle, directory, current,
+                                wildcard=self.fileMask, flags=self.fileMode)
         if value:
             self.SetValue(value)
 
@@ -52,28 +60,27 @@ class ToolsDialog(wx.Dialog):
     def __init__(self, parent, owner, items=None):
         wx.Dialog.__init__(self, parent, -1, _("Toolbar editor"),
                           style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
-        ADD_ID, REMOVE_ID, NAME_ID, LABEL_ID, ID_ID, CHECK_RADIO_ID, LIST_ID, \
+        ADD_ID, REMOVE_ID, LABEL_ID, ID_ID, CHECK_RADIO_ID, LIST_ID, \
                 ADD_SEP_ID, MOVE_UP_ID, MOVE_DOWN_ID, HELP_STR_ID, \
                 LONG_HELP_STR_ID, BITMAP1_ID, BITMAP2_ID \
-                = [wx.NewId() for i in range(14)]
+                = [wx.NewId() for i in range(13)]
 
         self._staticbox = wx.StaticBox(self, -1, _("Tool:"))
 
         self.owner = owner
-        
+
         self.tool_items = wx.ListCtrl(self, LIST_ID, style=wx.LC_REPORT | \
-                                     wx.LC_SINGLE_SEL|wx.SUNKEN_BORDER,
+                                     wx.LC_SINGLE_SEL|wx.BORDER_SUNKEN,
                                      size=(300, -1))
-        self.selected_index = -1 # index of the selected element in the 
-                                 # wxListCtrl 
+        self.selected_index = -1  # index of the selected element in the
+                                 # wxListCtrl
         self.tool_items.InsertColumn(0, _("Label"))
         self.tool_items.InsertColumn(1, _("Id"))
-        self.tool_items.InsertColumn(2, _("Normal Bitmap"))
-        self.tool_items.InsertColumn(3, _("Second Bitmap"))
+        self.tool_items.InsertColumn(2, _("Primary Bitmap"))
+        self.tool_items.InsertColumn(3, _("Disabled Bitmap"))
         self.tool_items.InsertColumn(4, _("Short Help"))
         self.tool_items.InsertColumn(5, _("Long Help"))
         self.tool_items.InsertColumn(6, _("Type"))
-        # ALB 2004-12-05
         self.tool_items.InsertColumn(7, _("Event Handler"))
 
         self.tool_items.SetColumnWidth(0, 100)
@@ -83,22 +90,20 @@ class ToolsDialog(wx.Dialog):
         self.tool_items.SetColumnWidth(5, 100)
         self.tool_items.SetColumnWidth(6, 150)
         self.tool_items.SetColumnWidth(7, 150)
-        
+
         # tool fields
         self.id = wx.TextCtrl(self, ID_ID)
         self.label = wx.TextCtrl(self, LABEL_ID)
         self.help_str = wx.TextCtrl(self, HELP_STR_ID)
         self.long_help_str = wx.TextCtrl(self, LONG_HELP_STR_ID)
-        # ALB 2004-12-05
         self.event_handler = wx.TextCtrl(self, -1)
-        import re
         self.handler_re = re.compile(r'^\s*\w*\s*$')
 
         self.bitmap1 = _MyBrowseButton(
-            self, BITMAP1_ID, labelText=_('Normal Bitmap'), buttonText='...',
+            self, BITMAP1_ID, labelText=_('Primary Bitmap'), buttonText='...',
             changeCallback=self.update_tool)
         self.bitmap2 = _MyBrowseButton(
-            self, BITMAP2_ID, labelText=_('Second Bitmap'), buttonText='...',
+            self, BITMAP2_ID, labelText=_('Disabled Bitmap'), buttonText='...',
             changeCallback=self.update_tool)
         self.check_radio = wx.RadioBox(
             self, CHECK_RADIO_ID, _("Type"),
@@ -117,6 +122,7 @@ class ToolsDialog(wx.Dialog):
         self.cancel = wx.Button(self, wx.ID_CANCEL, _("Cancel"))
 
         self.do_layout()
+
         # event handlers
         wx.EVT_BUTTON(self, ADD_ID, self.add_tool)
         wx.EVT_BUTTON(self, REMOVE_ID, self.remove_tool)
@@ -143,7 +149,7 @@ class ToolsDialog(wx.Dialog):
         self.bitmap1.Enable(False)
         self.bitmap2.Enable(False)
         self.check_radio.Enable(False)
-        
+
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer2 = wx.StaticBoxSizer(self._staticbox, wx.VERTICAL)
         self.label.SetSize((150, -1))
@@ -152,10 +158,7 @@ class ToolsDialog(wx.Dialog):
         self.long_help_str.SetSize((150, -1))
         self.event_handler.SetSize((150, -1))
         szr = wx.FlexGridSizer(0, 2)
-        if misc.check_wx_version(2, 5, 2):
-            flag = wx.FIXED_MINSIZE
-        else:
-            flag = 0
+        flag = wx.FIXED_MINSIZE
         label_flag = wx.ALIGN_CENTER_VERTICAL
         szr.Add(wx.StaticText(self, -1, _("Id   ")), flag=label_flag)
         szr.Add(self.id, flag=flag)
@@ -168,7 +171,6 @@ class ToolsDialog(wx.Dialog):
         szr.Add(wx.StaticText(self, -1, _("Event Handler  ")), flag=label_flag)
         szr.Add(self.event_handler, flag=flag)
         sizer2.Add(szr, 1, wx.ALL|wx.EXPAND, 5)
-        label_w = self.bitmap1.browseButton.GetTextExtent('...')[0]
         sizer2.Add(self.bitmap1, 0, wx.EXPAND)
         sizer2.Add(self.bitmap2, 0, wx.EXPAND)
         sizer2.Add(self.check_radio, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 4)
@@ -185,7 +187,7 @@ class ToolsDialog(wx.Dialog):
         sizer4.Add(self.move_down, 0, wx.LEFT|wx.RIGHT, 5)
         sizer3.Add(sizer4, 0, wx.ALIGN_CENTER|wx.ALL, 5)
         szr = wx.BoxSizer(wx.HORIZONTAL)
-        szr.Add(sizer3, 1, wx.ALL|wx.EXPAND, 5) 
+        szr.Add(sizer3, 1, wx.ALL|wx.EXPAND, 5)
         szr.Add(sizer2, 0, wx.TOP|wx.BOTTOM|wx.RIGHT, 5)
         sizer.Add(szr, 1, wx.EXPAND)
         sizer2 = wx.BoxSizer(wx.HORIZONTAL)
@@ -203,17 +205,18 @@ class ToolsDialog(wx.Dialog):
         """\
         Event handler called when the Add button is clicked
         """
-        index = self.selected_index = self.selected_index+1
+        index = self.selected_index = self.selected_index + 1
         if not self.tool_items.GetItemCount():
             for s in (self.label, self.id, self.help_str, self.long_help_str,
                       self.bitmap1, self.bitmap2, self.check_radio,
                       self.event_handler):
                 s.Enable(True)
-        if index < 0: index = self.tool_items.GetItemCount()
-        name, label, id, check_radio = "", "item", "", "0"
+        if index < 0:
+            index = self.tool_items.GetItemCount()
+        label, wid, check_radio = "item", "", "0"
         bitmap1, bitmap2, help_str, long_help_str = [""] * 4
         self.tool_items.InsertStringItem(index, label)
-        self.tool_items.SetStringItem(index, 1, id)
+        self.tool_items.SetStringItem(index, 1, wid)
         self.tool_items.SetStringItem(index, 2, bitmap1)
         self.tool_items.SetStringItem(index, 3, bitmap2)
         self.tool_items.SetStringItem(index, 4, help_str)
@@ -222,7 +225,7 @@ class ToolsDialog(wx.Dialog):
         self.tool_items.SetItemState(index, wx.LIST_STATE_SELECTED,
                                      wx.LIST_STATE_SELECTED)
         self.label.SetValue(label)
-        self.id.SetValue(id)
+        self.id.SetValue(wid)
         self.check_radio.SetSelection(int(check_radio))
         self.bitmap1.SetValue(bitmap1, False)
         self.bitmap2.SetValue(bitmap2, False)
@@ -240,8 +243,8 @@ class ToolsDialog(wx.Dialog):
                       self.bitmap1, self.bitmap2, self.check_radio,
                       self.event_handler):
                 s.Enable(True)
-        if index < 0: index = self.tool_items.GetItemCount() 
-        self.tool_items.InsertStringItem(index, '---')#label)
+        if index < 0: index = self.tool_items.GetItemCount()
+        self.tool_items.InsertStringItem(index, '---')
         for i in range(1, 5):
             self.tool_items.SetStringItem(index, i, '---')
         self.tool_items.SetItemState(index, wx.LIST_STATE_SELECTED,
@@ -250,7 +253,7 @@ class ToolsDialog(wx.Dialog):
     def show_tool(self, event):
         """\
         Event handler called when a tool in the list is selected
-        """        
+        """
         self.selected_index = index = event.GetIndex()
         get_item = self.tool_items.GetItem
         if not self.tool_items.GetItem(index, 2).m_text == '---':
@@ -272,7 +275,7 @@ class ToolsDialog(wx.Dialog):
         """\
         Event handler called when some of the properties of the current tool
         changes
-        """        
+        """
         set_item = self.tool_items.SetStringItem
         index = self.selected_index
         handler = self.event_handler.GetValue()
@@ -305,7 +308,7 @@ class ToolsDialog(wx.Dialog):
     def remove_tool(self, event):
         """\
         Event handler called when the Remove button is clicked
-        """        
+        """
         if self.selected_index >= 0:
             for s in (self.id, self.label, self.help_str, self.long_help_str,
                       self.event_handler):
@@ -329,6 +332,7 @@ class ToolsDialog(wx.Dialog):
         set_item = self.tool_items.SetStringItem
         add_item = self.tool_items.InsertStringItem
         index = [0]
+
         def add(tool):
             i = index[0]
             add_item(i, misc.wxstr(tool.label))
@@ -354,8 +358,10 @@ class ToolsDialog(wx.Dialog):
         returns the contents of self.tool_items as a list of tools that
         describes the contents of the ToolBar
         """
-        def get(i, j): return self.tool_items.GetItem(i, j).m_text
+        def get(i, j):
+            return self.tool_items.GetItem(i, j).m_text
         tools = []
+
         def add(index):
             label = get(index, 0)
             id = get(index, 1)
@@ -426,6 +432,7 @@ class ToolsProperty(Property):
     """\
     Property to edit the tools of an EditToolBar instance.
     """
+
     def __init__(self, owner, name, parent):
         Property.__init__(self, owner, name, parent)
         self.panel = None
@@ -437,110 +444,159 @@ class ToolsProperty(Property):
         edit_btn_id = wx.NewId()
         self.edit_btn = wx.Button(self.panel, edit_btn_id, _("Edit tools..."))
         sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(self.edit_btn, 1, wx.EXPAND|wx.ALIGN_CENTER|wx.TOP|wx.BOTTOM,
-                  4)
+        sizer.Add(self.edit_btn, 1,
+                  wx.EXPAND | wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, 4)
         self.panel.SetAutoLayout(1)
-        self.panel.SetSizer(sizer)
+        compat.SizerItem_SetSizer(self.panel, sizer)
         self.panel.SetSize(sizer.GetMinSize())
         wx.EVT_BUTTON(self.panel, edit_btn_id, self.edit_tools)
 
-    def bind_event(*args): pass
+    def bind_event(self, function):
+        pass
 
     def edit_tools(self, event):
         dialog = ToolsDialog(self.panel, self.owner,
                              items=self.owner.get_tools())
         if dialog.ShowModal() == wx.ID_OK:
             self.owner.set_tools(dialog.get_tools())
-            common.app_tree.app.saved = False # update the status of the app
+            common.app_tree.app.saved = False  # update the status of the app
 
     def write(self, outfile, tabs):
-        fwrite = outfile.write
-        fwrite('    ' * tabs + '<tools>\n')
+        inner_xml = StringIO.StringIO()
         for tool in self.owner[self.name][0]():
-            tool.write(outfile, tabs+1)
-        fwrite('    ' * tabs + '</tools>\n')
+            tool.write(inner_xml, tabs + 1)
+        stmt = common.format_xml_tag(
+            u'tools', inner_xml.getvalue(), tabs, is_xml=True)
+        outfile.write(stmt)
 
 # end of class ToolsProperty
 
 
-class EditToolBar(EditBase, PreviewMixin):
+class ToolsHandler(BaseXmlBuilderTagHandler):
+    itemattrs = ['label', 'id', 'short_help', 'long_help',
+                 'bitmap1', 'bitmap2', 'type', 'handler']
+
+    def __init__(self, owner):
+        super(ToolsHandler, self).__init__()
+        self.owner = owner
+        self.tools = []
+        self.curr_tool = None
+        self.curr_index = -1
+
+    def start_elem(self, name, attrs):
+        if name == 'tools':
+            return
+        if name == 'tool':
+            self.curr_tool = Tool()
+        else:
+            try:
+                self.curr_index = self.itemattrs.index(name)
+            except ValueError:
+                self.curr_index = -1
+                # just ignore the attributes we don't know
+
+    def end_elem(self, name):
+        if name == 'tool':
+            self.tools.append(self.curr_tool)
+        if name == 'tools':
+            self.owner.set_tools(self.tools)
+            return True
+
+    def char_data(self, data):
+        super(ToolsHandler, self).char_data(data)
+        if self.curr_index >= 0:
+            char_data = self.get_char_data()
+            setattr(self.curr_tool,
+                    self.itemattrs[self.curr_index], char_data)
+
+# end of class ToolsHandler
+
+
+class EditToolBar(EditBase, PreviewMixin, EditStylesMixin, BitmapMixin):
+    """\
+    Class to handle wxToolBar objects
+
+    @ivar pwidget: Parent widget
+    @type pwidget: wxToolBar | wxFrame
+
+    @ivar tools: List of Tool objects
+    @type tools: list[Tool]
+    """
+
     def __init__(self, name, klass, parent, property_window):
+
+        # Initialise parent classes
         custom_class = parent is None
-        EditBase.__init__(self, name, klass,
-                          parent, wx.NewId(), property_window,
-                          custom_class=custom_class, show=False)
-        self.base = 'wx.ToolBar'
-        
-        def nil(*args): return ()
-        self.tools = [] # list of Tool objects
-        self._tb = None # the real toolbar
-        self.style = 0
-        self.access_functions['style'] = (self.get_style, self.set_style)
-        self.style_pos  = [wx.TB_FLAT, wx.TB_DOCKABLE, wx.TB_3DBUTTONS]
-        if misc.check_wx_version(2, 3, 3):
-            self.style_pos += [wx.TB_TEXT, wx.TB_NOICONS, wx.TB_NODIVIDER,
-                               wx.TB_NOALIGN]
-        if misc.check_wx_version(2, 5, 0):
-            self.style_pos += [wx.TB_HORZ_LAYOUT, wx.TB_HORZ_TEXT]
- 
-        style_labels = ['#section#' + _('Style'), 'wxTB_FLAT', 'wxTB_DOCKABLE',
-                        'wxTB_3DBUTTONS']
-        if misc.check_wx_version(2, 3, 3):
-            style_labels += ['wxTB_TEXT', 'wxTB_NOICONS',
-                             'wxTB_NODIVIDER', 'wxTB_NOALIGN']
-        if misc.check_wx_version(2, 5, 0):
-            style_labels += ['wxTB_HORZ_LAYOUT', 'wxTB_HORZ_TEXT']
-        self.properties['style'] = CheckListProperty(self, 'style', None,
-                                                     style_labels)
-        self.bitmapsize = '16, 15'
-        self.access_functions['bitmapsize'] = (self.get_bitmapsize,
-                                               self.set_bitmapsize)
-        self.properties['bitmapsize'] = TextProperty(self, 'bitmapsize', None,
-                                                     can_disable=True, label=_("bitmapsize"))
-        self.margins = '0, 0'
-        self.access_functions['margins'] = (self.get_margins, self.set_margins)
-        self.properties['margins'] = TextProperty(self, 'margins', None,
-                                                  can_disable=True, label=_("margins"))
-        self.access_functions['tools'] = (self.get_tools, self.set_tools)
-        prop = self.properties['tools'] = ToolsProperty(self, 'tools', None)
-        self.packing = 1
-        self.access_functions['packing'] = (self.get_packing, self.set_packing)
-        self.properties['packing'] = SpinProperty(self, 'packing', None,
-                                                  r=(0, 100), can_disable=True, label=_("packing"))
-        self.separation = 5
-        self.access_functions['separation'] = (self.get_separation,
-                                               self.set_separation)
-        self.properties['separation'] = SpinProperty(
-            self, 'separation', None, r=(0, 100), can_disable=True, label=_("separation"))
-        # 2003-05-07 preview support
+        EditBase.__init__(self, name, 'wxToolBar', parent, wx.NewId(),
+                          property_window, custom_class=custom_class,
+                          show=False)
+        EditStylesMixin.__init__(self)
         PreviewMixin.__init__(self)
 
+        # initialise instance variables
+        self.tools = []
+        self.pwidget = None
+        self.widget = None
+
+        # initialise properties remaining staff
+        access = self.access_functions
+        properties = self.properties
+
+        access['style'] = (self.get_style, self.set_style)
+        properties['style'] = CheckListProperty(
+            self, 'style', self.widget_writer)
+
+        self.bitmapsize = '16, 15'
+        access['bitmapsize'] = (self.get_bitmapsize, self.set_bitmapsize)
+        properties['bitmapsize'] = TextProperty(
+            self, 'bitmapsize', can_disable=True,label=_("Bitmap size"))
+
+        self.margins = '0, 0'
+        access['margins'] = (self.get_margins, self.set_margins)
+        properties['margins'] = TextProperty(
+            self, 'margins', can_disable=True, label=_("Margins"))
+
+        access['tools'] = (self.get_tools, self.set_tools)
+        properties['tools'] = ToolsProperty(self, 'tools', None)
+
+        self.packing = 1
+        access['packing'] = (self.get_packing, self.set_packing)
+        properties['packing'] = SpinProperty(
+            self, 'packing', r=(0, 100), can_disable=True,
+            label=_("Packing"))
+
+        self.separation = 5
+        access['separation'] = (self.get_separation, self.set_separation)
+        properties['separation'] = SpinProperty(
+            self, 'separation', r=(0, 100), can_disable=True,
+            label=_("Separation"))
+
     def create_widget(self):
-        tb_style = wx.TB_HORIZONTAL|self.style
-        if wx.Platform == '__WXGTK__': tb_style |= wx.TB_DOCKABLE|wx.TB_FLAT
+        tb_style = wx.TB_HORIZONTAL | self.get_int_style()
+        if wx.Platform == '__WXGTK__':
+            tb_style |= wx.TB_DOCKABLE | wx.TB_FLAT
         if self.parent:
-            self.widget = self._tb = wx.ToolBar(
+            self.pwidget = self.widget = wx.ToolBar(
                 self.parent.widget, -1, style=tb_style)
-            self.parent.widget.SetToolBar(self.widget)
+            self.parent.widget.SetToolBar(self.pwidget)
         else:
             # "top-level" toolbar
-            self.widget = wx.Frame(None, -1, misc.design_title(self.name))
-            self.widget.SetClientSize((400, 30))
-            self._tb = wx.ToolBar(self.widget, -1, style=tb_style)
-            self.widget.SetToolBar(self._tb)
-            self.widget.SetBackgroundColour(self._tb.GetBackgroundColour())
-            import os
+            self.pwidget = wx.Frame(None, -1, misc.design_title(self.name))
+            self.pwidget.SetClientSize((400, 30))
+            self.widget = wx.ToolBar(self.pwidget, -1, style=tb_style)
+            self.pwidget.SetToolBar(self.widget)
+            self.pwidget.SetBackgroundColour(self.widget.GetBackgroundColour())
             icon = wx.EmptyIcon()
-            xpm = os.path.join(common.wxglade_path, 'icons', 'toolbar.xpm')
+            xpm = os.path.join(config.icons_path, 'toolbar.xpm')
             icon.CopyFromBitmap(misc.get_xpm_bitmap(xpm))
-            self.widget.SetIcon(icon)
-            wx.EVT_CLOSE(self.widget, lambda e: self.hide_widget())
-            wx.EVT_LEFT_DOWN(self._tb, self.on_set_focus)
+            self.pwidget.SetIcon(icon)
+            wx.EVT_CLOSE(self.pwidget, lambda e: self.hide_widget())
+            wx.EVT_LEFT_DOWN(self.widget, self.on_set_focus)
             if wx.Platform == '__WXMSW__':
                 # MSW isn't smart enough to avoid overlapping windows, so
                 # at least move it away from the 3 wxGlade frames
-                self.widget.CenterOnScreen()
-        wx.EVT_LEFT_DOWN(self.widget, self.on_set_focus)
+                self.pwidget.CenterOnScreen()
+        wx.EVT_LEFT_DOWN(self.pwidget, self.on_set_focus)
         # set the various property values
         prop = self.properties
         if prop['bitmapsize'].is_active():
@@ -551,7 +607,7 @@ class EditToolBar(EditBase, PreviewMixin):
             self.set_packing(self.packing, refresh=False)
         if prop['separation'].is_active():
             self.set_separation(self.separation, refresh=False)
-        self.set_tools(self.tools) # show the menus
+        self.set_tools(self.tools)  # show the menus
 
     def create_properties(self):
         EditBase.create_properties(self)
@@ -568,7 +624,7 @@ class EditToolBar(EditBase, PreviewMixin):
             sizer.Add(self.name_prop.panel, 0, wx.EXPAND)
             sizer.Add(self.klass_prop.panel, 0, wx.EXPAND)
             page.SetAutoLayout(1)
-            page.SetSizer(sizer)
+            compat.SizerItem_SetSizer(page, sizer)
         sizer.Add(self.properties['bitmapsize'].panel, 0, wx.EXPAND)
         sizer.Add(self.properties['margins'].panel, 0, wx.EXPAND)
         sizer.Add(self.properties['packing'].panel, 0, wx.EXPAND)
@@ -577,36 +633,20 @@ class EditToolBar(EditBase, PreviewMixin):
         sizer.Add(self.properties['tools'].panel, 0, wx.ALL|wx.EXPAND, 3)
         sizer.Layout()
         sizer.Fit(page)
-        w, h = page.GetClientSize()
+        h = page.GetClientSize()[1]
         self.notebook.AddPage(page, _("Common"))
         if self.parent is not None:
             self.property_window.Layout()
-            page.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
+            page.SetScrollbars(1, 5, 1, int(math.ceil(h / 5.0)))
         else:
             PreviewMixin.create_properties(self)
-        
+
     def __getitem__(self, key):
         return self.access_functions[key]
 
-    def get_style(self):
-        retval = [0] * len(self.style_pos)
-        try:
-            for i in range(len(self.style_pos)):
-                if (self.style & self.style_pos[i]) == self.style_pos[i]:
-                    retval[i] = 1
-        except AttributeError:
-            pass
-        return retval
-
-    def set_style(self, value, refresh=True):
-        value = self.properties['style'].prepare_value(value)
-        self.style = 0
-        for v in range(len(value)):
-            if value[v]:
-                self.style |= self.style_pos[v]
-        if self._tb:
-            self._tb.SetWindowStyleFlag(self.style)
-            if refresh: self._refresh_widget()
+    def set_style(self, value):
+        EditStylesMixin.set_style(self, value)
+        self._refresh_widget()
 
     def get_margins(self):
         return self.margins
@@ -618,9 +658,10 @@ class EditToolBar(EditBase, PreviewMixin):
             self.properties['margins'].set_value(self.margins)
         else:
             self.margins = value
-            if self._tb:
-                self._tb.SetMargins(margins)
-                if refresh: self._refresh_widget()
+            if self.widget:
+                self.widget.SetMargins(margins)
+                if refresh:
+                    self._refresh_widget()
 
     def get_packing(self):
         return self.packing
@@ -632,9 +673,10 @@ class EditToolBar(EditBase, PreviewMixin):
             self.properties['packing'].set_value(self.packing)
         else:
             self.packing = value
-            if self._tb:
-                self._tb.SetToolPacking(self.packing)
-                if refresh: self._refresh_widget()
+            if self.widget:
+                self.widget.SetToolPacking(self.packing)
+                if refresh:
+                    self._refresh_widget()
 
     def get_separation(self):
         return self.separation
@@ -646,13 +688,14 @@ class EditToolBar(EditBase, PreviewMixin):
             self.properties['separation'].set_value(self.separation)
         else:
             self.separation = value
-            if self._tb:
-                self._tb.SetToolSeparation(self.separation)
-                if refresh: self._refresh_widget()
+            if self.widget:
+                self.widget.SetToolSeparation(self.separation)
+                if refresh:
+                    self._refresh_widget()
 
     def get_bitmapsize(self):
         return self.bitmapsize
-        
+
     def set_bitmapsize(self, value, refresh=True):
         try:
             size = [int(t.strip()) for t in value.split(',')]
@@ -660,159 +703,99 @@ class EditToolBar(EditBase, PreviewMixin):
             self.properties['bitmapsize'].set_value(self.bitmapsize)
         else:
             self.bitmapsize = value
-            if self._tb:
-                self._tb.SetToolBitmapSize(size)
-                if refresh: self._refresh_widget()
+            if self.widget:
+                self.widget.SetToolBitmapSize(size)
+                if refresh:
+                    self._refresh_widget()
 
     def get_tools(self):
         return self.tools
 
     def set_tools(self, tools):
         self.tools = tools
-        if not self._tb: return # nothing left to do
-        while self._tb.DeleteToolByPos(0):
-            pass # clear the toolbar
+        if not self.widget:
+            return  # nothing left to do
+        self.widget.ClearTools()
         # now add all the tools
         for tool in self.tools:
-            if misc.streq(tool.id, '---'): # the tool is a separator
-                self._tb.AddSeparator()
+            if misc.streq(tool.id, '---'):  # the tool is a separator
+                self.widget.AddSeparator()
             else:
-                if tool.bitmap1:
-                    bmp1 = None
-                    if not (tool.bitmap1.startswith('var:') or
-                            tool.bitmap1.startswith('code:')):
-                        bmp1 = wx.Bitmap(
-                            misc.get_relative_path(misc.wxstr(tool.bitmap1)),
-                            wx.BITMAP_TYPE_ANY)
-                    if not bmp1 or not bmp1.Ok(): bmp1 = wx.EmptyBitmap(1, 1)
-                else:
-                    bmp1 = wx.NullBitmap
-                if tool.bitmap2:
-                    bmp2 = None
-                    if not (tool.bitmap2.startswith('var:') or
-                            tool.bitmap2.startswith('code:')):
-                        bmp2 = wx.Bitmap(
-                            misc.get_relative_path(misc.wxstr(tool.bitmap2)),
-                            wx.BITMAP_TYPE_ANY)
-                    if not bmp2 or not bmp2.Ok(): bmp2 = wx.EmptyBitmap(1, 1)
-                else:
-                    bmp2 = wx.NullBitmap
-                # signature of AddTool for 2.3.2.1:
-                # wxToolBarToolBase *AddTool(
-                #    int id, const wxBitmap& bitmap,
-                #    const wxBitmap& pushedBitmap, bool toggle = FALSE,
-                #    wxObject *clientData = NULL,
-                #    const wxString& shortHelpString = wxEmptyString,
-                #    const wxString& longHelpString = wxEmptyString)
-                if not misc.check_wx_version(2, 3, 3):
-                    # use the old signature, some of the entries are ignored
-                    self._tb.AddTool(wx.NewId(), bmp1, bmp2, tool.type == 1,
-                                     shortHelpString=\
-                                     misc.wxstr(tool.short_help),
-                                     longHelpString=misc.wxstr(tool.long_help))
-                else:
-                    kinds = [wx.ITEM_NORMAL, wx.ITEM_CHECK, wx.ITEM_RADIO]
-                    try:
-                        kind = kinds[int(tool.type)]
-                    except (ValueError, IndexError):
-                        kind = wx.ITEM_NORMAL
-                    self._tb.AddLabelTool(wx.NewId(), misc.wxstr(tool.label),
-                                          bmp1, bmp2, kind,
-                                          misc.wxstr(tool.short_help),
-                                          misc.wxstr(tool.long_help))
+                bmp1 = self.get_preview_obj_bitmap(tool.bitmap1)
+                bmp2 = self.get_preview_obj_bitmap(tool.bitmap2)
+                kinds = [wx.ITEM_NORMAL, wx.ITEM_CHECK, wx.ITEM_RADIO]
+                try:
+                    kind = kinds[int(tool.type)]
+                except (ValueError, IndexError):
+                    kind = wx.ITEM_NORMAL
+                self.widget.AddLabelTool(wx.NewId(), misc.wxstr(tool.label),
+                                         bmp1, bmp2, kind,
+                                         misc.wxstr(tool.short_help),
+                                         misc.wxstr(tool.long_help))
         # this is required to refresh the toolbar properly
         self._refresh_widget()
 
     def _refresh_widget(self):
-        self._tb.Realize()
-        self._tb.SetSize((-1, self._tb.GetBestSize()[1]))
+        self.widget.Realize()
+        self.widget.SetSize((-1, self.widget.GetBestSize()[1]))
         if self.parent:
             widget = self.parent.widget
             w, h = widget.GetClientSize()
-            widget.SetClientSize((w, h+1))
+            widget.SetClientSize((w, h + 1))
             widget.SetClientSize((w, h))
         else:
-            widget = self.widget
+            widget = self.pwidget
             w = widget.GetClientSize()[0]
-            h = self._tb.GetSize()[1] / 2
+            h = self.widget.GetSize()[1] / 2
             widget.SetClientSize((w, h))
-        
+
     def remove(self, *args, **kwds):
         if self.parent is not None:
             self.parent.properties['toolbar'].set_value(0)
-            if kwds.get('do_nothing', False): # and wxPlatform == '__WXGTK__':
+            if kwds.get('do_nothing', False):
                 # this probably leaks memory, but avoids segfaults
-                self.widget = None
+                self.pwidget = None
             else:
                 if self.parent.widget:
                     self.parent.widget.SetToolBar(None)
         else:
-            if self.widget:
-                self.widget.Destroy()
-                self.widget = None
+            if self.pwidget:
+                self.pwidget.Destroy()
+                self.pwidget = None
         EditBase.remove(self)
 
     def popup_menu(self, event):
         if self.parent is not None:
-            return # do nothing in this case
-        if self.widget:
-            if not self._rmenu:
-                REMOVE_ID, HIDE_ID = [wx.NewId() for i in range(2)]
-                self._rmenu = misc.wxGladePopupMenu(self.name)
-                misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
-                                 wx.ART_DELETE)
-                misc.append_item(self._rmenu, HIDE_ID, _('Hide')) 
-                def bind(method):
-                    return lambda e: misc.wxCallAfter(method)
-                wx.EVT_MENU(self.widget, REMOVE_ID, bind(self.remove))
-                wx.EVT_MENU(self.widget, HIDE_ID, bind(self.hide_widget))
-                
-            self.widget.PopupMenu(self._rmenu, event.GetPosition())
+            return  # do nothing in this case
+        super(EditToolBar, self).popup_menu(event)
+
+    def _create_popup_menu(self):
+        REMOVE_ID = wx.NewId()
+        HIDE_ID = wx.NewId()
+        self._rmenu = misc.wxGladePopupMenu(self.name)
+        misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
+                         wx.ART_DELETE)
+        misc.append_item(self._rmenu, HIDE_ID, _('Hide'))
+
+        def bind(method):
+            return lambda e: wx.CallAfter(method)
+
+        wx.EVT_MENU(self.pwidget, REMOVE_ID, bind(self.remove))
+        wx.EVT_MENU(self.pwidget, HIDE_ID, bind(self.hide_widget))
 
     def hide_widget(self, *args):
-        if self.widget and self.widget is not self._tb:
-            self.widget.Hide()
+        if self.pwidget and self.pwidget is not self.widget:
+            self.pwidget.Hide()
             common.app_tree.expand(self.node, False)
             common.app_tree.select_item(self.node.parent)
             common.app_tree.app.show_properties()
 
     def set_name(self, name):
         EditBase.set_name(self, name)
-        if self.widget is not self._tb:
-            self.widget.SetTitle(misc.design_title(misc.wxstr(self.name)))
+        if self.pwidget is not self.widget:
+            self.pwidget.SetTitle(misc.design_title(misc.wxstr(self.name)))
 
     def get_property_handler(self, name):
-        class ToolsHandler:
-            itemattrs = ['label', 'id', 'short_help', 'long_help',
-                         'bitmap1', 'bitmap2', 'type', 'handler']
-            def __init__(self, owner):
-                self.owner = owner
-                self.tools = []
-                self.curr_tool = None
-                self.curr_index = -1
-            def start_elem(self, name, attrs):
-                if name == 'tools': return
-                if name == 'tool':
-                    self.curr_tool = Tool()
-                else:
-                    try:
-                        self.curr_index = self.itemattrs.index(name)
-                    except ValueError:
-                        self.curr_index = -1
-                        pass # just ignore the attributes we don't know
-##                         from xml_parse import XmlParsingError
-##                         raise XmlParsingError, _("invalid tool attribute")
-            def end_elem(self, name):
-                if name == 'tool':
-                    self.tools.append(self.curr_tool)
-                if name == 'tools':
-                    self.owner.set_tools(self.tools)
-                    return True
-            def char_data(self, data):
-                if self.curr_index >= 0:
-                    setattr(self.curr_tool,
-                            self.itemattrs[self.curr_index], data)
-                
         if name == 'tools':
             return ToolsHandler(self)
         return None
@@ -845,38 +828,36 @@ def builder(parent, sizer, pos, number=[0]):
             szr.Fit(self)
             #self.SetSize((150, -1))
 
-        def undo(self):
-            if number[0] > 0:
-                number[0] -= 1
-
         def __getitem__(self, value):
             if value == 'class':
                 def set_klass(c): self.klass = c
-                return (lambda : self.klass, set_klass)
+                return lambda : self.klass, set_klass
     # end of inner class
 
     dialog = Dialog()
-    if dialog.ShowModal() == wx.ID_CANCEL:
-        # cancel the operation
-        dialog.undo()
-        dialog.Destroy()
+    res = dialog.ShowModal()
+    klass = dialog.klass
+    dialog.Destroy()
+    if res != wx.ID_OK:
+        if number[0] > 0:
+            number[0] -= 1
         return
-    
+
     name = 'toolbar_%d' % (number[0] or 1)
     while common.app_tree.has_name(name):
         number[0] += 1
         name = 'toolbar_%d' % number[0]
 
-    tb = EditToolBar(name, dialog.klass, parent, common.property_panel)
+    tb = EditToolBar(name, klass, parent, common.property_panel)
     tb.node = Tree.Node(tb)
     common.app_tree.add(tb.node)
     tb.show_widget(True)
     tb.show_properties()
-    
+
 
 def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """\
-    factory to build EditMenuBar objects from an xml file
+    factory to build EditMenuBar objects from a XML file
     """
     name = attrs.get('name')
     if parent is not None:
@@ -897,8 +878,7 @@ def initialize():
     initialization function for the module: returns a wxBitmapButton to be
     added to the main palette.
     """
-    cwx = common.widgets_from_xml
-    cwx['EditToolBar'] = xml_builder
+    common.widgets_from_xml['EditToolBar'] = xml_builder
     common.widgets['EditToolBar'] = builder
-    
-    return common.make_object_button('EditToolBar', 'icons/toolbar.xpm', 1)
+
+    return common.make_object_button('EditToolBar', 'toolbar.xpm', 1)
